//+------------------------------------------------------------------+
//|                                         Translator positions.mq5 |
//|                                            Copyright 2010, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
#property version   "1.00"
input string subfolder="MetaTrader 4";// subfolder name (with MetaTrader 4 client terminal) in \MQL5\Files\ folder
int cnt_command=1;
int  TotalSymbols;
string Symbols[];    // symbol names
double SymPosVol[];  // symbol volumes
int    SymPosType[]; // position type (-1 if there is no position)
double SymPosSL[];   // position Stop Loss 
double SymPosTP[];   // position Take Profit
                     // previous values
double prevSymPosVol[];  // previous position volume
int    prevSymPosType[]; // previous position type (or -1 if there is no position)
double prevSymPosSL[];   // previous value of Stop Loss level (or 0)
double prevSymPosTP[];   // previous value of Take Profit level (or 0)
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- total symbols in MarketWatch   
   TotalSymbols=SymbolsTotal(true); // call SymbolsTotal(true) for MarketWatch symbols only
//---  resize array for symbols
   ArrayResize(Symbols,TotalSymbols);
//---  resize array for position volumes
   ArrayResize(SymPosVol,TotalSymbols);
   ArrayInitialize(SymPosVol,0.0);
//---  resize arrays for position types
   ArrayResize(SymPosType,TotalSymbols);
   ArrayInitialize(SymPosType,-1);
//---  resize array for Stop Loss levels
   ArrayResize(SymPosSL,TotalSymbols);
   ArrayInitialize(SymPosSL,0.0);
//---  resize array for Take Profit levels
   ArrayResize(SymPosTP,TotalSymbols);
   ArrayInitialize(SymPosTP,0.0);
//---  resize arrays for previous values
   ArrayResize(prevSymPosVol,TotalSymbols);
   ArrayResize(prevSymPosType,TotalSymbols);
   ArrayResize(prevSymPosSL,TotalSymbols);
   ArrayResize(prevSymPosTP,TotalSymbols);
//--- fill symbols with names from MarketWatch
   for(int i=0;i<TotalSymbols;i++)
      Symbols[i]=SymbolName(i,true); //i index in the list// true because of MarketWatch symbols only

//--- set timer with frequency 1 sec.
   EventSetTimer(1);
//--- 
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- clear comments
   Comment("");
//--- kill timer
   EventKillTimer();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTimer()
  {
//--- get positions
   get_positions();
//--- if positions has changed, save position
   if(compare_positions())saves_positions();
  }
//+------------------------------------------------------------------+
//| Custom functions                                                 |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| get positions                                                    |
//+------------------------------------------------------------------+
void get_positions()
  {
//--- symbols loop
   for(int i=0;i<TotalSymbols;i++)
     {
      //--- select position
      if(PositionSelect(Symbols[i]))
        {
         //--- get position volume
         SymPosVol[i]=(double)PositionGetDouble(POSITION_VOLUME);
         //--- get position type
         SymPosType[i]=(int)PositionGetInteger(POSITION_TYPE);
         //--- get position stop loss
         SymPosSL[i]=(double)PositionGetDouble(POSITION_SL);
         //--- get position take profit
         SymPosTP[i]=(double)PositionGetDouble(POSITION_TP);
        }
      //--- if there isn't position, filling the values with 0 
      else {SymPosVol[i]=0.0; SymPosType[i]=-1;SymPosSL[i]=0.0;SymPosTP[i]=0.0;}
     }
  }
//+------------------------------------------------------------------+
//| Compare positions                                                |
//| Returns true if the data are different                           |
//+------------------------------------------------------------------+
bool compare_positions()
  {
//--- loop all symbols
   for(int i=0;i<TotalSymbols;i++)
     {
      //--- compare volumes
      if(prevSymPosVol[i]!=SymPosVol[i])return(true);
      //--- compare types
      if(prevSymPosType[i]!=SymPosType[i])return(true);
      //--- compare Stop Loss levels
      if(prevSymPosSL[i]!=SymPosSL[i])return(true);
      //--- compare Take Profit levels
      if(prevSymPosTP[i]!=SymPosTP[i])return(true);
     }
//--- if all positions are equal
   return(false);
  }
//+------------------------------------------------------------------+
//| Save positions                                                   |
//+------------------------------------------------------------------+
void saves_positions()
  {
//--- saves positions on all symbols
   for(int i=0;i<TotalSymbols;i++)
     {
      //--- save position volume
      prevSymPosVol[i]=SymPosVol[i];
      //--- save position type
      prevSymPosType[i]=SymPosType[i];
      //--- save position Stop Loss
      prevSymPosSL[i]=SymPosSL[i];
      //--- save position Take Profit
      prevSymPosTP[i]=SymPosTP[i];
     }
//--- write position states to file
   WriteFile();
  }
//+------------------------------------------------------------------+
//| Calculates symbol volume step                                    |
//+------------------------------------------------------------------+
int DigitsStepLot(string symbol)
  {
//--- 1/x (volume step)
   double Dig=1.0/SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);
   int res=0;
//--- conversion of 1/x to digits
   if(Dig>1)res=1;
   if(Dig>10)res=2;
   if(Dig>100)res=3;
   if(Dig>1000)res=4;
   return(res);
  }
//+------------------------------------------------------------------+
//| write positions states to a csv file                             |
//+------------------------------------------------------------------+
void WriteFile(string folder="Translator positions")
  {
//--- open file for writing, shared read mode
   int han=FileOpen(subfolder+"\\experts\\files\\"+folder+".csv",FILE_WRITE|FILE_SHARE_READ|FILE_ANSI,",");
   if(han!=INVALID_HANDLE)// file opened successfully
     {
      string com=""; // comment
      FileWrite(han,(string)cnt_command); cnt_command++;
      for(int i=0;i<TotalSymbols;i++)
        {
         //--- write position state on symbol, specified in Symbols[i]
         FileWrite(han,
                   //--- position symbol
                   Symbols[i],
                   //--- position volume
                   DoubleToString(prevSymPosVol[i],DigitsStepLot(Symbols[i])),
                   //--- position type
                   (string)prevSymPosType[i],
                   //--- position stop loss
                   DoubleToString(prevSymPosSL[i],(int)SymbolInfoInteger(Symbols[i],SYMBOL_DIGITS)),
                   //--- position take profit
                   DoubleToString(prevSymPosTP[i],(int)SymbolInfoInteger(Symbols[i],SYMBOL_DIGITS))
                   );
         //--- comment
         com=com+"\n"+
             //--- position symbol
             Symbols[i]+"  [ "+
             //--- position volume
             DoubleToString(prevSymPosVol[i],DigitsStepLot(Symbols[i]))+" ]  [ "+
             //--- position type
             (string)prevSymPosType[i]+" ]  [ "+
             //--- position stop loss
             DoubleToString(prevSymPosSL[i],(int)SymbolInfoInteger(Symbols[i],SYMBOL_DIGITS))+" ]  [ "+
             //--- position take profit
             DoubleToString(prevSymPosTP[i],(int)SymbolInfoInteger(Symbols[i],SYMBOL_DIGITS))+" ]"
             ;
        }
      //--- close file
      FileClose(han);
      //--- print comment
      Comment((string)(cnt_command-1),com);
     }
//--- if file open has failed
   else Print("File open has failed"+subfolder+"\\"+folder+".csv, error",GetLastError());
  }
//+------------------------------------------------------------------+
